#include <cxxtest/TestSuite.h>

#include "Error.h"
#include "MockErrno.h"

class ErrorTestSuite : public CxxTest::TestSuite
{
public:
	void test_code_and_errno()
	{
        MockErrno mockErrno;

        mockErrno.nextErrno = 21;

		Error error1( Error::NOT_ENOUGH_MEMORY );
		TS_ASSERT_EQUALS( error1.code(), Error::NOT_ENOUGH_MEMORY );
        TS_ASSERT_EQUALS( error1.errno(), 21 );
        TS_ASSERT_EQUALS( error1.message(), "" );

        mockErrno.nextErrno = 23;

		Error error2( (Error::Code)166 );
		TS_ASSERT_EQUALS( error2.code(), (Error::Code)166 );
        TS_ASSERT_EQUALS( error2.errno(), 23 );
        TS_ASSERT_EQUALS( error2.message(), "" );
	}

	void test_code_and_errno__with_message()
	{
        MockErrno mockErrno;

        mockErrno.nextErrno = 21;

		Error error1( Error::NOT_ENOUGH_MEMORY, "out of memeory error" );
		TS_ASSERT_EQUALS( error1.code(), Error::NOT_ENOUGH_MEMORY );
        TS_ASSERT_EQUALS( error1.errno(), 21 );
        TS_ASSERT_EQUALS( error1.message(), "out of memeory error" );

        mockErrno.nextErrno = 23;

		Error error2( (Error::Code)166, "Other, strange error" );
		TS_ASSERT_EQUALS( error2.code(), (Error::Code)166 );
        TS_ASSERT_EQUALS( error2.errno(), 23 );
        TS_ASSERT_EQUALS( error2.message(), "Other, strange error" );
	}
};

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
