#ifndef __BProgram_h__
#define __BProgram_h__

#include "AutoEngine.h"
#include "IBThread.h"
#include "T/ParametersFactory.h"
#include "ThreadStateGraph.h"
#include "Vector.h"
#include <unistd.h>

class BProgram
{
public:
    BProgram( const String &parameterFile ) : _engine( NULL )
    {
        T::loadParameters( parameterFile );
        _engine = T::createEngine();
    }

    ~BProgram()
    {
        if ( _engine )
        {
            T::discardEngine( _engine );
            _engine = NULL;
        }
    }

    void addThread( IBThread &thread )
    {
        _threads.append( &thread );
    }

    void addLocalStateGraph( const String &threadName, ThreadStateGraph &threadStateGraph )
    {
        _engine->addLocalStateGraph( threadName, threadStateGraph );
    }

    void addForeignStateGraph( const String &threadName, ThreadStateGraph &threadStateGraph )
    {
        _engine->addForeignStateGraph( threadName, threadStateGraph );
    }

    void runProgram()
    {
        runThreads();
    }

    void runEngine()
    {
        _engine->run();
    }

private:
    IEngine *_engine;
    Vector<IBThread *> _threads;

    void runThreads()
    {
        for ( unsigned i = 0; i < _threads.size(); ++i )
        {
            _threads[i]->run();
        }
    }
};

#endif // __BProgram_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
