#ifndef __CoreCommunicator_h__
#define __CoreCommunicator_h__

#include "AutoServerSocket.h"
#include "AutoSocket.h"
#include "CoreMessage.h"
#include "EngineConfiguration.h"
#include "HeapData.h"
#include "ICoreCommunicator.h"
#include "IWeaver.h"
#include "T/ParametersFactory.h"
#include <arpa/inet.h>
#include <cstdlib>

class CoreCommunicator : public ICoreCommunicator
{
public:
    CoreCommunicator( ISelector &selector, IWeaver &weaver ) :
        _selector( selector ), _serverSocket( NULL ), _socket( NULL ), _weaver( weaver )
    {
        coreOne() ? initCoreOne() : initCoreTwo();
    }

    ~CoreCommunicator()
    {
        unregisterFromSelector();
        discardSocketIfNeeded();
        discardServerSocketIfNeeded();
    }

    void handleAcceptedSocket( ISocket &socket )
	{
        discardServerSocketIfNeeded();
        _socket = &socket;

        registerToSelector();
    }

    int descriptor()
    {
        return _socket->descriptor();
    }

    void canRead()
    {
        HeapData data;
        _socket->receive( data );

        unsigned offset = 0;

        while ( offset < data.size() )
        {
            CoreMessage coreMessage( ConstSimpleData( (char *)data.data() + offset, data.size() - offset ) );
            _weaver.incomingCoreMessage( coreMessage );
            offset += coreMessage.serializedSize();
        }
    }

    void send( CoreMessage &coreMessage )
    {
        HeapData serialized;
        coreMessage.serialize( serialized );

        _socket->send( serialized );
    }

private:
    ISelector &_selector;
    IServerSocket *_serverSocket;
    ISocket *_socket;
    IWeaver &_weaver;

    void initCoreOne()
    {
        _serverSocket = T::createServerSocket( _selector, T::coreCommunicationPort(), *this );
    }

    void initCoreTwo()
    {
        _socket = T::createSocketIpPort( inet_addr( EngineConfiguration::LOOPBACK_IP ), T::coreCommunicationPort() );
        registerToSelector();
    }

    void discardServerSocketIfNeeded()
    {
        if ( _serverSocket )
        {
            T::discardServerSocket( _serverSocket );
            _serverSocket = NULL;
        }
    }

    void discardSocketIfNeeded()
    {
        if ( _socket )
        {
            T::discardSocket( _socket );
            _socket = NULL;
        }
    }

    static bool coreOne()
    {
        enum {
            CORE_ONE = 1,
        };

        return T::coreNumber() == CORE_ONE;
    }

    void registerToSelector()
    {
        _selector.registerToSelector( *this );
    }

    void unregisterFromSelector()
    {
        _selector.unregisterFromSelector( *this );
    }
};

#endif // __CoreCommunicator_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
