#ifndef __CoreMessage_h__
#define __CoreMessage_h__

#include "ConstSimpleData.h"
#include "HeapData.h"
#include "String.h"

class CoreMessage
{
public:
    CoreMessage( unsigned long long synchronizationIndex, const String &threadName, const String &chosenLabel ) :
        _synchronizationIndex( synchronizationIndex ), _threadName( threadName ), _chosenLabel( chosenLabel )
    {
    }

    bool operator==( const CoreMessage &other )
    {
        return
            threadName() == other.threadName() &&
            synchronizationIndex() == other.synchronizationIndex();
    }

    unsigned serializedSize() const
    {
        return _serializedSize;
    }

    void serialize( HeapData &serialized )
    {
        addIndex( serialized );
        addName( serialized );
        addLabel( serialized );
    }

    CoreMessage( const ConstSimpleData &serialized ) : _serializedSize( 0 )
    {
        extractIndex( serialized );
        extractName( serialized );
        extractLabel( serialized );
    }

    unsigned long long synchronizationIndex() const
    {
        return _synchronizationIndex;
    }

    String threadName() const
    {
        return _threadName;
    }

    String chosenLabel() const
    {
        return _chosenLabel;
    }

private:
    unsigned long long _synchronizationIndex;
    String _threadName;
    String _chosenLabel;
    unsigned _serializedSize;

    void addIndex( HeapData &serialized )
    {
        serialized = ConstSimpleData( &_synchronizationIndex, sizeof(_synchronizationIndex) );
    }

    void addName( HeapData &serialized )
    {
        unsigned nameLength = _threadName.length();
        serialized += ConstSimpleData( &nameLength, sizeof(nameLength) );

        ConstSimpleData nameString( _threadName.ascii(), _threadName.length() );
        serialized += nameString;
    }

    void addLabel( HeapData &serialized )
    {
        unsigned labelLength = _chosenLabel.length();
        serialized += ConstSimpleData( &labelLength, sizeof(labelLength) );

        ConstSimpleData labelString( _chosenLabel.ascii(), _chosenLabel.length() );
        serialized += labelString;
    }

    void extractIndex( const ConstSimpleData &serialized )
    {
        memcpy( &_synchronizationIndex, serialized.asChar(), sizeof(_synchronizationIndex) );
        _serializedSize = sizeof(_synchronizationIndex);
    }

    void extractName( const ConstSimpleData &serialized )
    {
        unsigned nameLength;
        memcpy( &nameLength, serialized.asChar() + _serializedSize, sizeof(nameLength) );

        _serializedSize += sizeof(unsigned);

        _threadName = String( serialized.asChar() + _serializedSize, nameLength );

        _serializedSize += nameLength;
    }

    void extractLabel( const ConstSimpleData &serialized )
    {
        unsigned labelLength;
        memcpy( &labelLength, serialized.asChar() + _serializedSize, sizeof(labelLength) );

        _serializedSize += sizeof(unsigned);

        _chosenLabel = String( serialized.asChar() + _serializedSize, labelLength );

        _serializedSize += labelLength;
    }
};

#endif // __CoreMessage_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
