#ifndef __Engine_h__
#define __Engine_h__

#include "AutoCoreCommunicator.h"
#include "AutoSelector.h"
#include "AutoServerSocket.h"
#include "AutoWeaver.h"
#include "EngineConfiguration.h"
#include "Error.h"
#include "Event.h"
#include "IEngine.h"
#include "T/ParametersFactory.h"
#include "T/Truth.h"

class Engine : public IEngine
{
public:
	static void main()
	{
		Engine().entryPoint();
	}

	Engine() : _weaver( _selector, *this ),
               _serverSocket( _selector, T::bThreadManagementPort(), _weaver ),
               _coreCommunicator( _selector, _weaver )
	{
	}

    void addLocalStateGraph( const String &threadName, ThreadStateGraph &threadStateGraph )
    {
        _weaver->addLocalStateGraph( threadName, threadStateGraph );
    }

    void addForeignStateGraph( const String &threadName, ThreadStateGraph &threadStateGraph )
    {
        _weaver->addForeignStateGraph( threadName, threadStateGraph );
    }

	void entryPoint()
	{
		while ( T::alwaysTrue() )
		{
			_selector->select();
		}
	}

    void sendCoreMessage( CoreMessage &coreMessage )
    {
        // printf( "Engine: sending core message to other core\n" );
        // fflush( 0 );

        _coreCommunicator->send( coreMessage );
    }

private:
	AutoSelector _selector;
	AutoWeaver _weaver;
	AutoServerSocket _serverSocket;
    AutoCoreCommunicator _coreCommunicator;
};

#endif // __Engine_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
