#ifndef __EventSelector_h__
#define __EventSelector_h__

#include "Event.h"
#include "Vector.h"

class EventSelector
{
public:
    static Event select( Vector<Event> &requested, Vector<Event> &blocked )
    {
        Vector<Event> enabled;

        for ( unsigned i = 0; i < requested.size(); ++i )
        {
            if ( !blocked.exists( requested[i] ) )
            {
                enabled.append( requested[i] );
            }
        }

        if ( enabled.empty() )
        {
            throw Error( Error::SELECTION_DEADLOCKED );
        }

        return determinsticChoice( enabled );
    }

    static Event determinsticChoice( Vector<Event> &options )
    {
        static unsigned lastTriggered = 0;
        static bool firstRound = true;

        if ( firstRound )
        {
            lastTriggered = pickSmallestEvent( options );
            firstRound = false;
        }
        else if ( haveEventGreaterThan( options, lastTriggered ) )
        {
            lastTriggered = pickLowestGreaterThan( options, lastTriggered );
        }
        else
        {
            lastTriggered = pickSmallestEvent( options );
        }

        return Event( lastTriggered );
    }

    static unsigned pickSmallestEvent( Vector<Event> &options )
    {
        unsigned index = 0;

        for ( unsigned i = 1; i < options.size(); ++i )
        {
            if ( options[i].type() < options[index].type() )
            {
                index = i;
            }
        }

        return options[index].type();
    }

    static unsigned pickLowestGreaterThan( Vector<Event> &options, unsigned value )
    {
        bool foundFirst = false;
        unsigned index;

        for ( unsigned i = 0; i < options.size(); ++i )
        {
            if ( foundFirst && ( options[i].type() < options[index].type() ) && ( options[i].type() > value ) )
            {
                index = i;
            }

            if ( !foundFirst && ( options[i].type() > value ) )
            {
                index = i;
                foundFirst = true;
            }
        }

        return options[index].type();
    }

    static bool haveEventGreaterThan( Vector<Event> &options, unsigned value )
    {
        for ( unsigned i = 0; i < options.size(); ++i )
        {
            if ( options[i].type() > value )
            {
                return true;
            }
        }

        return false;
    }
};

#endif // __EventSelector_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
