#ifndef __ThreadState_h__
#define __ThreadState_h__

#include "AtomicPropositions.h"
#include "Event.h"
#include "String.h"
#include "Transition.h"
#include "UniqueVector.h"

struct HistoryItem
{
    HistoryItem( const Event &event, unsigned environmentConfiguration ) :
        _event( event ), _environmentConfiguration( environmentConfiguration )
    {
    }

    Event _event;
    unsigned _environmentConfiguration;
};

struct ThreadState
{
    String _label;
    Vector<HistoryItem> _history;
    Vector<Event> _requestedEvents;
    Vector<Event> _watchedEvents;
    Vector<Event> _blockedEvents;
    Vector<Transition> _successors;
    Vector<Transition> _predecessors;
    AtomicPropositions _atomicPropositions;

    void clear()
    {
        _label = "";

        _history.clear();
        _requestedEvents.clear();
        _watchedEvents.clear();
        _blockedEvents.clear();
        _successors.clear();
        _predecessors.clear();
    }

    String labelOfSuccessor( Event event )
    {
        for ( unsigned i = 0; i < _successors.size(); ++i )
        {
            if ( _successors[i]._event == event )
            {
                return _successors[i]._state;
            }
        }

        return "";
    }

    bool deterministicEvent( Event event )
    {
        unsigned transitionCount = 0;

        for ( unsigned i = 0; i < _successors.size(); ++i )
        {
            if ( _successors[i]._event == event )
            {
                ++transitionCount;
            }
        }

        return transitionCount < 2;
    }

    Vector<String> allSuccessors()
    {
        UniqueVector<String> successors;

        for ( unsigned i = 0; i < _successors.size(); ++i )
        {
            successors.append( _successors[i]._state );
        }

        return successors;
    }

    void removeSuccessor( const String &label )
    {
        unsigned i = 0;

        while ( i < _successors.size() )
        {
            if ( _successors[i]._state == label )
            {
                _successors.eraseAt( i );
            }
            else
            {
                ++i;
            }
        }
    }

    void removePredecessor( const String &label )
    {
        unsigned i = 0;

        while ( i < _predecessors.size() )
        {
            if ( _predecessors[i]._state == label )
            {
                _predecessors.eraseAt( i );
            }
            else
            {
                ++i;
            }
        }
    }
};

#endif // __ThreadState_h__

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
