#ifndef __MockEngine_h__
#define __MockEngine_h__

#include "CoreMessage.h"
#include "IEngine.h"
#include "TestLog.h"

class MockEngine : public IEngine
{
public:
	MockEngine()
	{
		wasCreated = false;
		wasDiscarded = false;

        runWasCalled = false;
        coreMessageSent = false;
    }

	bool wasCreated;
	bool wasDiscarded;

	void mockConstructor()
	{
		TS_ASSERT( !wasCreated );
		wasCreated = true;

        TS_ASSERT( TestLog::happened( "Parameters loaded" ) );
	}

	void mockDestructor()
	{
		TS_ASSERT( wasCreated );
		TS_ASSERT( !wasDiscarded );
		wasDiscarded = true;
	}

    bool runWasCalled;

    void run()
    {
        runWasCalled = true;

        TestLog::log( "Engine was run" );
    }

    void entryPoint()
    {
    }

    String lastThreadName;
    ThreadStateGraph *lastLocalStateGraph;

    void addLocalStateGraph( const String &threadName, ThreadStateGraph &threadStateGraph )
    {
        lastThreadName = threadName;
        lastLocalStateGraph = &threadStateGraph;
    }

    ThreadStateGraph *lastForeignStateGraph;

    void addForeignStateGraph( const String &threadName, ThreadStateGraph &threadStateGraph )
    {
        lastThreadName = threadName;
        lastForeignStateGraph = &threadStateGraph;
    }

    String lastSentLabel;
    unsigned long long lastSentMessageIndex;
    bool coreMessageSent;

    void sendCoreMessage( CoreMessage &coreMessage )
    {
        coreMessageSent = true;
        lastSentLabel = coreMessage.chosenLabel();
        lastSentMessageIndex = coreMessage.synchronizationIndex();
    }
};

#endif // __MockEngine_h__

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
