#ifndef __MockWeaver_h__
#define __MockWeaver_h__

#include "CoreMessage.h"
#include "Event.h"
#include "IEngine.h"
#include "ISelector.h"
#include "IWeaver.h"
#include "Vector.h"

class MockWeaver : public IWeaver
{
public:
	MockWeaver()
	{
		wasCreated = false;
		wasDiscarded = false;

        bSyncWasCalled = false;
	}

	bool wasCreated;
	bool wasDiscarded;

    ISelector *lastSelector;
    IEngine *lastEngine;

	void mockConstructor( ISelector &selector, IEngine &engine )
	{
		TS_ASSERT( !wasCreated );
		wasCreated = true;

        lastSelector = &selector;
        lastEngine = &engine;
	}

	void mockDestructor()
	{
		TS_ASSERT( wasCreated );
		TS_ASSERT( !wasDiscarded );
		wasDiscarded = true;
	}

	void handleAcceptedSocket( ISocket & )
	{
	}

    Vector<Event> lastRequested;
    Vector<Event> lastWatched;
    Vector<Event> lastBlocked;
    String lastLabel;
    IThreadListener *lastThread;
    bool bSyncWasCalled;
    unsigned long long lastSynchronizationIndex;

    void bSync( IThreadListener *thread,
                const Vector<Event> &requested,
                const Vector<Event> &watched,
                const Vector<Event> &blocked,
                const String &label,
                unsigned long long synchronizationIndex )
    {
        lastRequested = requested;
        lastWatched = watched;
        lastBlocked = blocked;
        lastThread = thread;
        lastLabel = label;
        lastSynchronizationIndex = synchronizationIndex;

        bSyncWasCalled = true;
    }

    String lastThreadName;
    ThreadStateGraph *lastLocalStateGraph;

    void addLocalStateGraph( const String &threadName, ThreadStateGraph &threadStateGraph )
    {
        lastThreadName = threadName;
        lastLocalStateGraph = &threadStateGraph;
    }

    ThreadStateGraph *lastForeignStateGraph;

    void addForeignStateGraph( const String &threadName, ThreadStateGraph &threadStateGraph )
    {
        lastThreadName = threadName;
        lastForeignStateGraph = &threadStateGraph;
    }

    Vector<CoreMessage> lastCoreMessages;

    void incomingCoreMessage( CoreMessage &coreMessage )
    {
        lastCoreMessages.append( coreMessage );
    }
};

#endif // __MockWeaver_h__

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
