#include <cxxtest/TestSuite.h>

#include "ConstSimpleData.h"
#include "RealMalloc.h"
#include "CoreMessage.h"

class MockForCoreMessage :
    public RealMalloc
{
};

class CoreMessageTestSuite : public CxxTest::TestSuite
{
public:
    MockForCoreMessage *mock;

    void setUp()
    {
        TS_ASSERT( mock = new MockForCoreMessage );
    }

    void tearDown()
    {
        TS_ASSERT_THROWS_NOTHING( delete mock );
    }

    void test_serialize()
    {
        CoreMessage coreMessage( 0x0123456789ABCDEFull, "jon", "hello world" );

        HeapData serialized;

        TS_ASSERT_THROWS_NOTHING( coreMessage.serialize( serialized ) );

        unsigned char expectedData[] = {
            0xef, 0xcd, 0xab, 0x89, 0x67, 0x45, 0x23, 0x01,
            0x03, 0x00, 0x00, 0x00,
            0x6a, 0x6f, 0x6e,
            0x0b, 0x00, 0x00, 0x00,
            0x68, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x77, 0x6f, 0x72, 0x6c, 0x64
        };

        TS_ASSERT_EQUALS( serialized.size(), sizeof(expectedData) );
        TS_ASSERT_SAME_DATA( serialized.data(), expectedData, sizeof(expectedData) );
    }

    void test_unserialize()
    {
        unsigned char serializedData[] = {
            0xef, 0xcd, 0xab, 0x89, 0x67, 0x45, 0x23, 0x01,
            0x03, 0x00, 0x00, 0x00,
            0x6a, 0x6f, 0x6e,
            0x0b, 0x00, 0x00, 0x00,
            0x68, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x77, 0x6f, 0x72, 0x6c, 0x64
        };

        ConstSimpleData serialized( serializedData, sizeof(serializedData) );

        CoreMessage coreMessage( serialized );

        TS_ASSERT_EQUALS( coreMessage.synchronizationIndex(), 0x0123456789ABCDEFull );
        TS_ASSERT_EQUALS( coreMessage.threadName(), "jon" );
        TS_ASSERT_EQUALS( coreMessage.chosenLabel(), "hello world" );

        TS_ASSERT_EQUALS( coreMessage.serializedSize(), sizeof(serializedData) );
    }

    void test_equality()
    {
        CoreMessage c1( 17, "hey", "there" );
        CoreMessage c2( 17, "hey", "bla" );
        CoreMessage c3( 18, "hey", "there" );
        CoreMessage c4( 17, "bye", "there" );
        CoreMessage c5( 17, "hey", "there" );

        TS_ASSERT_EQUALS( c1, c1 );
        TS_ASSERT_EQUALS( c1, c2 );
        TS_ASSERT_DIFFERS( c1, c3 );
        TS_ASSERT_DIFFERS( c1, c4 );
        TS_ASSERT_EQUALS( c1, c5 );
    }
};

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
