#ifndef __Adviser_h__
#define __Adviser_h__

#include "BThread.h"
#include "IEvent.h"
#include "Point.h"
#include "String.h"
#include "T/Truth.h"

class Adviser : public BThread
{
public:
    Adviser( Point locationOne, Point locationTwo ) :
        BThread( "Adviser" ), _locationOne( locationOne ), _locationTwo( locationTwo )
    {
    }

    void entryPoint()
    {
        while ( true )
        {
            clearAll();
            blockIfNeeded();
            waitForMove();
            updatePositions();
        }
    }

private:
    Vector<Event> _requested;
    Vector<Event> _watched;
    Vector<Event> _blocked;

    Point _locationOne;
    Point _locationTwo;

    void clearAll()
    {
        _requested.clear();
        _watched.clear();
        _blocked.clear();
    }

    void waitForMove()
    {
        _watched = IEvent::allEvents();
        bSync( _requested, _watched, _blocked, "Waiting" );
    }

    void updatePositions()
    {
        if ( lastEvent() == IEvent::ONE_MOVE_EAST )
        {
            ++_locationOne._x;
        }
        else if ( lastEvent() == IEvent::ONE_MOVE_WEST )
        {
            --_locationOne._x;
        }
        else if ( lastEvent() == IEvent::ONE_MOVE_NORTH )
        {
            ++_locationOne._y;
        }
        else if ( lastEvent() == IEvent::ONE_MOVE_SOUTH )
        {
            --_locationOne._y;
        }
        else if ( lastEvent() == IEvent::TWO_MOVE_EAST )
        {
            ++_locationTwo._x;
        }
        else if ( lastEvent() == IEvent::TWO_MOVE_WEST )
        {
            --_locationTwo._x;
        }
        else if ( lastEvent() == IEvent::TWO_MOVE_NORTH )
        {
            ++_locationTwo._y;
        }
        else if ( lastEvent() == IEvent::TWO_MOVE_SOUTH )
        {
            --_locationTwo._y;
        }
    }

    void blockIfNeeded()
    {
        if ( ( _locationOne._x == _locationTwo._x + 1 ) && ( _locationOne._y == _locationTwo._y ) )
        {
            _blocked.append( IEvent::ONE_MOVE_WEST );
            _blocked.append( IEvent::TWO_MOVE_EAST );
        }
        else if ( ( _locationOne._x == _locationTwo._x - 1 ) && ( _locationOne._y == _locationTwo._y ) )
        {
            _blocked.append( IEvent::ONE_MOVE_EAST );
            _blocked.append( IEvent::TWO_MOVE_WEST );
        }
        else if ( ( _locationOne._y == _locationTwo._y + 1 ) && ( _locationOne._x == _locationTwo._x ) )
        {
            _blocked.append( IEvent::ONE_MOVE_SOUTH );
            _blocked.append( IEvent::TWO_MOVE_NORTH );
        }
        else if ( ( _locationOne._y == _locationTwo._y - 1 ) && ( _locationOne._x == _locationTwo._x ) )
        {
            _blocked.append( IEvent::ONE_MOVE_NORTH );
            _blocked.append( IEvent::TWO_MOVE_SOUTH );
        }

        if ( _blocked.size() > 0 )
        {
            printf( "\t\t\tAdviser: Blocking!\n" );
            fflush( 0 );
        }
    }
};

#endif // __Adviser_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
