#include "BProgram.h"
#include "Car1.h"
#include "ThreadStateGraph.h"
#include <cstdio>
#include <cstdlib>

void initializeRandomness()
{
    srand( time( NULL ) );
}

enum {
    PROGRAM_NAME = 0,
    CONFIG_FILE,
    NUM_OF_PARAMETERS,
};

void usage()
{
    printf( "\n\nUsage: <program name> <configuration file>\n" );
    fflush( 0 );
}

void prepareStateGraphOne( ThreadStateGraph &threadStateGraph )
{
    Event zero( IEvent::ZERO );
    Event one( IEvent::ONE );

    threadStateGraph._events.append( zero );
    threadStateGraph._events.append( one );

    threadStateGraph._initialState = "A";

    ThreadState a;
    ThreadState b;

    a._label = "A";
    a._requestedEvents.append( zero );
    a._successors.append( Transition( zero, "A" ) );
    a._successors.append( Transition( zero, "B" ) );

    b._label = "B";
    b._watchedEvents.append( zero );
    b._watchedEvents.append( one );

    b._successors.append( Transition( zero, "B" ) );
    b._successors.append( Transition( one, "A" ) );

    threadStateGraph._graph["A"] = a;
    threadStateGraph._graph["B"] = b;
}

void prepareStateGraphTwo( ThreadStateGraph &threadStateGraph )
{
    Event zero( IEvent::ZERO );
    Event one( IEvent::ONE );

    threadStateGraph._events.append( zero );
    threadStateGraph._events.append( one );

    threadStateGraph._initialState = "C";

    ThreadState c;
    ThreadState d;
    ThreadState e;

    c._label = "C";
    c._requestedEvents.append( zero );
    c._successors.append( Transition( zero, "D" ) );
    c._successors.append( Transition( zero, "E" ) );

    d._label = "D";
    d._requestedEvents.append( zero );
    d._successors.append( Transition( zero, "C" ) );

    e._label = "E";
    e._requestedEvents.append( one );
    e._successors.append( Transition( one, "C" ) );

    threadStateGraph._graph["C"] = c;
    threadStateGraph._graph["D"] = d;
    threadStateGraph._graph["E"] = e;
}

int main( int argc, char **argv )
{
    if ( argc != NUM_OF_PARAMETERS )
    {
        usage();
        return 1;
    }

    try
    {
        initializeRandomness();

        BProgram bProgram( argv[CONFIG_FILE] );

        ThreadStateGraph graphOne;
        prepareStateGraphOne( graphOne );
        bProgram.addLocalStateGraph( "Car1", graphOne );

        ThreadStateGraph graphTwo;
        prepareStateGraphTwo( graphTwo );
        bProgram.addForeignStateGraph( "Car2", graphTwo );

        bProgram.runEngine();

        Car1 car1;

        sleep( 1 );

        bProgram.addThread( car1 );

        sleep( 1 );

        bProgram.runProgram();

        sleep( 100 );
    }
    catch ( const Error &e )
    {
        printf( "Main Thread: Error caught. Code: %u. Errno: %i\n", e.code(), e.errno() );
        fflush( 0 );
    }

	return 0;
}

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
